package cn.com.qmth.scancentral.service.impl;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.apache.commons.collections4.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.github.jeffreyning.mybatisplus.service.MppServiceImpl;
import com.qmth.boot.core.exception.ParameterException;
import com.qmth.boot.core.fss.store.FileStore;

import cn.com.qmth.scancentral.bean.ImportSubjectDomain;
import cn.com.qmth.scancentral.bean.SubjectConfigDomain;
import cn.com.qmth.scancentral.bean.User;
import cn.com.qmth.scancentral.dao.SubjectDao;
import cn.com.qmth.scancentral.entity.SubjectEntity;
import cn.com.qmth.scancentral.service.BatchPaperService;
import cn.com.qmth.scancentral.service.FileService;
import cn.com.qmth.scancentral.service.StudentService;
import cn.com.qmth.scancentral.service.SubjectService;
import cn.com.qmth.scancentral.util.Calculator;
import cn.com.qmth.scancentral.util.FileUtil;
import cn.com.qmth.scancentral.vo.SubjectConfigVo;
import cn.com.qmth.scancentral.vo.examinfo.SubjectConfig;
import cn.com.qmth.scancentral.vo.subject.ScanProgressVo;
import cn.com.qmth.scancentral.vo.subject.SubjectBreachVo;
import cn.com.qmth.scancentral.vo.subject.SubjectCustStatusVo;
import cn.com.qmth.scancentral.vo.subject.SubjectScanProgressVo;

@Service
public class SubjectServiceImpl extends MppServiceImpl<SubjectDao, SubjectEntity> implements SubjectService {

    @Autowired
    private StudentService studentService;

    @Autowired
    private FileService fileService;

    @Autowired
    private FileStore fileStore;

    @Autowired
    private BatchPaperService batchPaperService;

    @Transactional
    @Override
    public int cleanByExamId(Long examId, String subjectCode) {
        QueryWrapper<SubjectEntity> wrapper = new QueryWrapper<>();
        LambdaQueryWrapper<SubjectEntity> lw = wrapper.lambda();
        lw.eq(SubjectEntity::getExamId, examId);
        lw.eq(SubjectEntity::getCode, subjectCode);
        return this.baseMapper.delete(wrapper);
    }

    @Override
    public List<SubjectEntity> listByExamId(Long examId) {
        QueryWrapper<SubjectEntity> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(SubjectEntity::getExamId, examId);
        queryWrapper.lambda().orderByDesc(SubjectEntity::getCode);
        return this.list(queryWrapper);
    }

    @Override
    public SubjectEntity findByExamIdAndCode(Long examId, String code) {
        QueryWrapper<SubjectEntity> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(SubjectEntity::getExamId, examId);
        queryWrapper.lambda().eq(SubjectEntity::getCode, code);
        return this.getOne(queryWrapper);
    }

    @Override
    public int countByExamId(Long examId) {
        QueryWrapper<SubjectEntity> wrapper = new QueryWrapper<>();
        LambdaQueryWrapper<SubjectEntity> lw = wrapper.lambda();
        lw.eq(SubjectEntity::getExamId, examId);
        return this.count(wrapper);
    }

    @Transactional
    @Override
    public int importSubject(List<ImportSubjectDomain> subjects) {
        Map<Long, Set<String>> map = new HashMap<Long, Set<String>>();
        List<SubjectEntity> list = this.list();
        for (SubjectEntity subjectEntity : list) {
            Set<String> set = map.get(subjectEntity.getExamId());
            if (set == null) {
                set = new HashSet<String>();
            }
            set.add(subjectEntity.getCode());
            map.put(subjectEntity.getExamId(), set);
        }

        List<SubjectEntity> saveList = new ArrayList<SubjectEntity>();
        for (ImportSubjectDomain importSubjectDomain : subjects) {
            Set<String> set = map.get(importSubjectDomain.getExamId());
            if (set != null) {
                if (!set.contains(importSubjectDomain.getSubjectCode())) {
                    SubjectEntity subjectEntity = new SubjectEntity(importSubjectDomain.getExamId(),
                            importSubjectDomain.getSubjectCode(), importSubjectDomain.getSubjectName());
                    set.add(importSubjectDomain.getSubjectCode());
                    saveList.add(subjectEntity);
                }
            } else {
                set = new HashSet<String>();
                SubjectEntity subjectEntity = new SubjectEntity(importSubjectDomain.getExamId(),
                        importSubjectDomain.getSubjectCode(), importSubjectDomain.getSubjectName());
                set.add(importSubjectDomain.getSubjectCode());
                saveList.add(subjectEntity);
            }
            map.put(importSubjectDomain.getExamId(), set);
        }
        this.saveOrUpdateBatchByMultiId(saveList);
        return saveList.size();
    }

    @Transactional
    @Override
    public SubjectConfigVo config(User user, SubjectConfigDomain domain) {
        SubjectEntity s = this.findByExamIdAndCode(domain.getExamId(), domain.getSubjectCode());
        if (s == null) {
            throw new ParameterException("找不到对应的科目");
        }
        s.setPaperTypeBarcodeContent(domain.getPaperTypeBarcodeContent());
        this.saveOrUpdateByMultiId(s);
        return new SubjectConfigVo(s);
    }

    @Override
    public List<SubjectConfig> listConfigByExamId(Long examId) {
        QueryWrapper<SubjectEntity> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(SubjectEntity::getExamId, examId);
        queryWrapper.lambda().isNotNull(SubjectEntity::getPaperTypeBarcodeContent);
        List<SubjectEntity> list = this.list(queryWrapper);
        List<SubjectConfig> result = new ArrayList<SubjectConfig>();
        for (SubjectEntity subjectEntity : list) {
            result.add(new SubjectConfig(subjectEntity));
        }
        return result;
    }

    @Override
    public ScanProgressVo scanProgress(Long examId, String subjectCode) {
        ScanProgressVo ret = new ScanProgressVo();
        List<SubjectScanProgressVo> subjects = studentService.scanProgress(examId, subjectCode);
        ret.setSubjects(subjects);
        ret.setTotal(new SubjectScanProgressVo());
        if (CollectionUtils.isNotEmpty(subjects)) {
            Calendar now = Calendar.getInstance();
            long endTime = now.getTimeInMillis();
            now.add(Calendar.MINUTE, -1);
            long startTime = now.getTimeInMillis();
            for (SubjectScanProgressVo vo : subjects) {
                vo.setScannedCount(vo.getStudentCount() - vo.getUnexistCount());
                vo.setProgress(Calculator.percentage(vo.getScannedCount(), vo.getStudentCount(), 2));
                Integer count = batchPaperService.getScanStudentCount(examId, subjectCode, startTime, endTime);
                if (count == null || count == 0) {
                    vo.setEstimation("-");
                } else {
                    vo.setEstimation(Calculator.divide(
                            Calculator.multiply(Calculator.divide(count, endTime - startTime), vo.getUnexistCount()),
                            3600000) + "h");
                }
            }
            int totalUnScanned = studentService.getUnscannedCountByExam(examId);
            int totalCount = studentService.getCountByExam(examId);
            ret.getTotal().setStudentCount(totalCount);
            ret.getTotal().setUnexistCount(totalUnScanned);
            ret.getTotal().setProgress(
                    Calculator.percentage(ret.getTotal().getScannedCount(), ret.getTotal().getStudentCount(), 2));
            Integer count = batchPaperService.getScanStudentCount(examId, null, startTime, endTime);
            if (count == null || count == 0) {
                ret.getTotal().setEstimation("-");
            } else {
                ret.getTotal().setEstimation(
                        Calculator.divide(Calculator.multiply(Calculator.divide(count, endTime - startTime),
                                ret.getTotal().getUnexistCount()), 3600000) + "h");
            }
        }
        return ret;
    }

    @Override
    public List<SubjectBreachVo> breachList(Long examId) {
        List<SubjectBreachVo> ret = new ArrayList<>();
        List<SubjectEntity> subjects = listByExamId(examId);
        if (CollectionUtils.isNotEmpty(subjects)) {
            for (SubjectEntity e : subjects) {
                SubjectBreachVo vo = new SubjectBreachVo();
                ret.add(vo);
                vo.setSubjectCode(e.getCode());
                vo.setSubjectName(e.getName());
                String url = fileService.getBreachInfoUri(examId, e.getCode());
                try {
                    if (fileStore.exist(url)) {
                        vo.setBreachCount(Integer.valueOf(FileUtil.readFileContent(fileStore.read(url)).trim()));
                    } else {
                        vo.setBreachCount(0);
                    }
                } catch (Exception e1) {
                    throw new RuntimeException("文件访问异常", e1);
                }
            }
        }
        return ret;
    }

    @Override
    public List<SubjectCustStatusVo> custStatusList(Long examId) {
        List<SubjectCustStatusVo> ret = new ArrayList<>();
        List<SubjectEntity> subjects = listByExamId(examId);
        if (CollectionUtils.isNotEmpty(subjects)) {
            for (SubjectEntity e : subjects) {
                SubjectCustStatusVo vo = new SubjectCustStatusVo();
                ret.add(vo);
                vo.setSubjectCode(e.getCode());
                vo.setSubjectName(e.getName());
                String url = fileService.getCustStatusInfoUri(examId, e.getCode());
                try {
                    if (fileStore.exist(url)) {
                        vo.setCustStatusCount(Integer.valueOf(FileUtil.readFileContent(fileStore.read(url)).trim()));
                    } else {
                        vo.setCustStatusCount(0);
                    }
                } catch (Exception e1) {
                    throw new RuntimeException("文件访问异常", e1);
                }
            }
        }
        return ret;
    }

}
